@@ -30,21 +30,43 @@ from zipfile import ZipFile, ZIP_DEFLATED, BadZipfile
 
 # package imports
 from openpyxl.shared.exc import OpenModeError, InvalidFileException
-from openpyxl.shared.ooxml import ARC_SHARED_STRINGS, ARC_CORE, ARC_APP, \
-        ARC_WORKBOOK, PACKAGE_WORKSHEETS, ARC_STYLE
+from openpyxl.shared.ooxml import (ARC_SHARED_STRINGS, ARC_CORE, ARC_WORKBOOK,
+                                   PACKAGE_WORKSHEETS, ARC_STYLE, ARC_THEME)
+from openpyxl.shared.compat import unicode, file
 from openpyxl.workbook import Workbook, DocumentProperties
 from openpyxl.reader.strings import read_string_table
 from openpyxl.reader.style import read_style_table
-from openpyxl.reader.workbook import read_sheets_titles, read_named_ranges, \
-        read_properties_core, get_sheet_ids, read_excel_base_date
+from openpyxl.reader.workbook import (read_sheets_titles, read_named_ranges,
+        read_properties_core, read_excel_base_date, get_sheet_ids)
 from openpyxl.reader.worksheet import read_worksheet
 from openpyxl.reader.iter_worksheet import unpack_worksheet
+# Use exc_info for Python 2 compatibility with "except Exception[,/ as] e"
+from sys import exc_info
+
+try:
+    # Python 2
+    unicode
+except NameError:
+    # Python 3
+    unicode = str
+
+def repair_central_directory(zipFile):
+    ''' trims trailing data from the central directory 
+    code taken from http://stackoverflow.com/a/7457686/570216, courtesy of Uri Cohen
+    '''
+    f = open(zipFile, 'r+b')
+    data = f.read().decode("utf-8")
+    pos = data.find('\x50\x4b\x05\x06') # End of central directory signature  
+    if (pos > 0):
+        f.seek(pos + 22)   # size of 'ZIP end of central directory record' 
+        f.truncate()
+        f.close()
 
 def load_workbook(filename, use_iterators=False):
     """Open the given filename and return the workbook
 
-    :param filename: the path to open
-    :type filename: string
+    :param filename: the path to open or a file-like object
+    :type filename: string or a file-like object open in binary mode c.f., :class:`zipfile.ZipFile`
 
     :param use_iterators: use lazy load for cells
     :type use_iterators: bool
@@ -58,7 +80,15 @@ def load_workbook(filename, use_iterators=False):
 
     """
 
-    if isinstance(filename, file):
+    try:
+        # Python 2
+        is_file_instance = isinstance(filename, file)
+    except NameError:
+        # Python 3
+        from io import BufferedReader
+        is_file_instance = isinstance(filename, BufferedReader)
+
+    if is_file_instance:
         # fileobject must have been opened with 'rb' flag
         # it is required by zipfile
         if 'b' not in filename.mode:
@@ -66,7 +96,15 @@ def load_workbook(filename, use_iterators=False):
 
     try:
         archive = ZipFile(filename, 'r', ZIP_DEFLATED)
-    except (BadZipfile, RuntimeError, IOError, ValueError), e:
+    except BadZipfile:
+        try:
+            repair_central_directory(filename)
+            archive = ZipFile(filename, 'r', ZIP_DEFLATED)
+        except BadZipfile:
+            e = exc_info()[1]
+            raise InvalidFileException(unicode(e))
+    except (BadZipfile, RuntimeError, IOError, ValueError):
+        e = exc_info()[1]
         raise InvalidFileException(unicode(e))
     wb = Workbook()
 
@@ -75,7 +113,8 @@ def load_workbook(filename, use_iterators=False):
 
     try:
         _load_workbook(wb, archive, filename, use_iterators)
-    except KeyError, e:
+    except KeyError:
+        e = exc_info()[1]
         raise InvalidFileException(unicode(e))
 
     archive.close()
@@ -95,9 +134,10 @@ def _load_workbook(wb, archive, filename, use_iterators):
         string_table = read_string_table(archive.read(ARC_SHARED_STRINGS))
     except KeyError:
         string_table = {}
+    wb.loaded_theme = archive.read(ARC_THEME)
     style_table = read_style_table(archive.read(ARC_STYLE))
 
-    wb.properties.excel_base_date = read_excel_base_date(xml_source = archive.read(ARC_WORKBOOK))
+    wb.properties.excel_base_date = read_excel_base_date(xml_source=archive.read(ARC_WORKBOOK))
 
     # get worksheets
     wb.worksheets = []  # remove preset worksheet
