@@ -1,6 +1,6 @@
 # file openpyxl/style.py
 
-# Copyright (c) 2010 openpyxl
+# Copyright (c) 2010-2011 openpyxl
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -21,7 +21,7 @@
 # THE SOFTWARE.
 #
 # @license: http://www.opensource.org/licenses/mit-license.php
-# @author: Eric Gazoni
+# @author: see AUTHORS file
 
 """Style and formatting option tracking."""
 
@@ -32,6 +32,7 @@ try:
 except ImportError:
     from md5 import md5
 
+from openpyxl.shared.compat import any
 
 class HashableObject(object):
     """Define how to hash property classes."""
@@ -307,7 +308,7 @@ class NumberFormat(HashableObject):
         48: '##0.0E+0',
         49: '@', }
     _BUILTIN_FORMATS_REVERSE = dict(
-            [(value, key) for key, value in _BUILTIN_FORMATS.iteritems()])
+            [(value, key) for key, value in _BUILTIN_FORMATS.items()])
 
     __fields__ = ('_format_code',
                   '_format_index')
