@@ -1,6 +1,6 @@
 # file openpyxl/writer/styles.py
 
-# Copyright (c) 2010 openpyxl
+# Copyright (c) 2010-2011 openpyxl
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -21,7 +21,7 @@
 # THE SOFTWARE.
 #
 # @license: http://www.opensource.org/licenses/mit-license.php
-# @author: Eric Gazoni
+# @author: see AUTHORS file
 
 """Write the shared style table."""
 
@@ -40,17 +40,18 @@ class StyleWriter(object):
     def _get_style_list(self, workbook):
         crc = {}
         for worksheet in workbook.worksheets:
-            for style in worksheet._styles.values():
+            uniqueStyles = dict((id(style), style) for style in worksheet._styles.values()).values()
+            for style in uniqueStyles:
                 crc[hash(style)] = style
         self.style_table = dict([(style, i + 1) \
             for i, style in enumerate(crc.values())])
-        sorted_styles = sorted(self.style_table.iteritems(), \
+        sorted_styles = sorted(self.style_table.items(), \
             key=lambda pair:pair[1])
         return [s[0] for s in sorted_styles]
 
     def get_style_by_hash(self):
         return dict([(hash(style), id) \
-            for style, id in self.style_table.iteritems()])
+            for style, id in self.style_table.items()])
 
     def write_table(self):
         number_format_table = self._write_number_formats()
@@ -88,7 +89,14 @@ class StyleWriter(object):
                 table[hash(st.font)] = str(index)
                 font_node = SubElement(fonts, 'font')
                 SubElement(font_node, 'sz', {'val':str(st.font.size)})
-                SubElement(font_node, 'color', {'rgb':str(st.font.color.index)})
+                if str(st.font.color.index).split(':')[0] == 'theme': # strip prefix theme if marked as such
+                    if str(st.font.color.index).split(':')[2]:
+                        SubElement(font_node, 'color', {'theme':str(st.font.color.index).split(':')[1],
+                                                        'tint':str(st.font.color.index).split(':')[2]})
+                    else:
+                        SubElement(font_node, 'color', {'theme':str(st.font.color.index).split(':')[1]})
+                else:
+                    SubElement(font_node, 'color', {'rgb':str(st.font.color.index)})
                 SubElement(font_node, 'name', {'val':st.font.name})
                 SubElement(font_node, 'family', {'val':'2'})
                 # Don't write the 'scheme' element because it appears to prevent
@@ -98,6 +106,9 @@ class StyleWriter(object):
                     SubElement(font_node, 'b')
                 if st.font.italic:
                     SubElement(font_node, 'i')
+                if st.font.underline == 'single':
+                    SubElement(font_node, 'u')
+
                 index += 1
 
         fonts.attrib["count"] = str(index)
@@ -119,10 +130,23 @@ class StyleWriter(object):
                 if hash(st.fill.fill_type) != hash(style.DEFAULTS.fill.fill_type):
                     node = SubElement(fill, 'patternFill', {'patternType':st.fill.fill_type})
                     if hash(st.fill.start_color) != hash(style.DEFAULTS.fill.start_color):
-
-                        SubElement(node, 'fgColor', {'rgb':str(st.fill.start_color.index)})
+                        if str(st.fill.start_color.index).split(':')[0] == 'theme': # strip prefix theme if marked as such
+                            if str(st.fill.start_color.index).split(':')[2]:
+                                SubElement(node, 'fgColor', {'theme':str(st.fill.start_color.index).split(':')[1],
+                                                             'tint':str(st.fill.start_color.index).split(':')[2]})
+                            else:
+                                SubElement(node, 'fgColor', {'theme':str(st.fill.start_color.index).split(':')[1]})
+                        else:
+                            SubElement(node, 'fgColor', {'rgb':str(st.fill.start_color.index)})
                     if hash(st.fill.end_color) != hash(style.DEFAULTS.fill.end_color):
-                        SubElement(node, 'bgColor', {'rgb':str(st.fill.start_color.index)})
+                        if str(st.fill.end_color.index).split(':')[0] == 'theme': # strip prefix theme if marked as such
+                            if str(st.fill.end_color.index).split(':')[2]:
+                                SubElement(node, 'bgColor', {'theme':str(st.fill.end_color.index).split(':')[1],
+                                                             'tint':str(st.fill.end_color.index).split(':')[2]})
+                            else:
+                                SubElement(node, 'bgColor', {'theme':str(st.fill.end_color.index).split(':')[1]})
+                        else:
+                            SubElement(node, 'bgColor', {'rgb':str(st.fill.end_color.index)})
                 index += 1
 
         fills.attrib["count"] = str(index)
@@ -149,8 +173,19 @@ class StyleWriter(object):
                 # caution: respect this order
                 for side in ('left', 'right', 'top', 'bottom', 'diagonal'):
                     obj = getattr(st.borders, side)
-                    node = SubElement(border, side, {'style':obj.border_style})
-                    SubElement(node, 'color', {'rgb':str(obj.color.index)})
+                    if obj.border_style is None or obj.border_style == 'none':
+                        node = SubElement(border, side)
+                        attrs = {}
+                    else:
+                        node = SubElement(border, side, {'style':obj.border_style})
+                        if str(obj.color.index).split(':')[0] == 'theme': # strip prefix theme if marked as such
+                            if str(obj.color.index).split(':')[2]:
+                                SubElement(node, 'color', {'theme':str(obj.color.index).split(':')[1],
+                                                                'tint':str(obj.color.index).split(':')[2]})
+                            else:
+                                SubElement(node, 'color', {'theme':str(obj.color.index).split(':')[1]})
+                        else:
+                            SubElement(node, 'color', {'rgb':str(obj.color.index)})
                 index += 1
 
         borders.attrib["count"] = str(index)
@@ -188,7 +223,7 @@ class StyleWriter(object):
 
             if hash(st.fill) != hash(style.DEFAULTS.fill):
                 vals['fillId'] = fills_table[hash(st.fill)]
-                vals['applyFillId'] = '1'
+                vals['applyFill'] = '1'
 
             if st.number_format != style.DEFAULTS.number_format:
                 vals['numFmtId'] = '%d' % number_format_table[st.number_format]
@@ -208,6 +243,11 @@ class StyleWriter(object):
 
                     if hash(st.alignment.wrap_text) != hash(style.DEFAULTS.alignment.wrap_text):
                         alignments['wrapText'] = '1'
+                    
+                    if st.alignment.text_rotation > 0:
+                        alignments['textRotation'] = '%s' % st.alignment.text_rotation
+                    elif st.alignment.text_rotation < 0:
+                        alignments['textRotation'] = '%s' % (90 - st.alignment.text_rotation)
 
                 SubElement(node, 'alignment', alignments)
 
